@@ -1,4 +1,4 @@
-/* $Id: acconfig.h 1.7 Wed, 05 Jan 2000 11:19:33 -0800 cengiz $
+/* $Id: acconfig.h 1.8 Thu, 09 Nov 2000 15:50:57 -0800 cengiz $
 // 
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -111,6 +111,9 @@
 /* Define if we accept/recvfrom/... uses socklen_t instead of size_t as arg */
 #undef ACCEPT_USES_SIZE_T
 
+/* Define if malloc.h has memalign definition */
+#undef HAVE_MEMALIGN_DEFINITION
+
 @BOTTOM@
 
 #ifdef __cplusplus
