@@ -1,4 +1,4 @@
-//  $Id: f_cisco.cc 1.10.1.5.1.50.1.4 Mon, 09 Apr 2001 10:38:04 -0700 cengiz $
+//  $Id: f_cisco.cc 1.10.1.5.1.50.1.5 Fri, 03 Aug 2001 15:30:24 -0700 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -390,7 +390,7 @@ void CiscoConfig::printREASno(ostream& out, const RangeList &no) {
    else {
       out << "_";
       pi = no.ranges.head();
-      put_par = ! no.ranges.isSingleton() || pi->high != pi->low;
+      put_par = ! no.ranges.isSingleton() || pi->high != pi->low || inTilda;
       if (put_par)
 	 if (reSplittable)
 	    if (inTilda)
@@ -530,7 +530,7 @@ void CiscoConfig::printRE(ostream &s, const regexp &r, int aclID, bool permit){
    out << ends;
 
    int lineLen = strlen(out.str());
-   if (lineLen < 250 & ! hasTilda) {
+   if (lineLen < 250 && ! hasTilda) {
       char *p = out.str();
       for (char *q = strchr(p, '@'); q; q = strchr(q, '@'))
 	 *q = '(';
@@ -922,7 +922,8 @@ bool CiscoConfig::printNeighbor(int import,
 
    if (!routeMapGenerated && (!import || ! prefixListGenerated)) {
       // yes we are conciously printing an "not any" policy
-      cout << "!\nroute-map " << mapName << " deny 1\n";
+      cout << "!\nno route-map " << mapName << "\n";
+      cout << "!\nroute-map " << mapName << " deny " << mapNumbersStartAt << "\n";
       routeMapGenerated = true;
    }
    
@@ -974,12 +975,12 @@ void CiscoConfig::exportP(ASt asno, IPAddr *addr,
       ne = NormalExpression::evaluate(fa->filter, peerAS);
 
       if (eliminateDupMapParts) {
-	 NormalExpression *ne2;
-	 ne2 = new NormalExpression(*ne);
-	 ne2->do_not();
-	 done.do_and(*ne2);
+	 NormalExpression ne2(*ne);
+	 NormalExpression done2(done);
+	 done2.do_not();
+	 ne2.do_and(done2);
 
-	 if (! done.isEmpty())
+	 if (! ne2.isEmpty())
 	    last = print(ne, fa->action, EXPORT);
 
 	 done.do_or(*ne);
@@ -1025,12 +1026,12 @@ void CiscoConfig::importP(ASt asno, IPAddr *addr,
       ne = NormalExpression::evaluate(fa->filter, peerAS);
 
       if (eliminateDupMapParts) {
-	 NormalExpression *ne2;
-	 ne2 = new NormalExpression(*ne);
-	 ne2->do_not();
-	 done.do_and(*ne2);
+	 NormalExpression ne2(*ne);
+	 NormalExpression done2(done);
+	 done2.do_not();
+	 ne2.do_and(done2);
 
-	 if (! done.isEmpty())
+	 if (! ne2.isEmpty())
 	    last = print(ne, fa->action, IMPORT);
 
 	 done.do_or(*ne);
@@ -1075,13 +1076,12 @@ void CiscoConfig::static2bgp(ASt asno, IPAddr *addr) {
       ne = NormalExpression::evaluate(fa->filter, asno);
 
       if (eliminateDupMapParts) {
-	 NormalExpression *ne2;
-	 ne2 = new NormalExpression(*ne);
-	 ne2->do_not();
-	 done.do_and(*ne2);
-	 delete ne2;
+	 NormalExpression ne2(*ne);
+	 NormalExpression done2(done);
+	 done2.do_not();
+	 ne2.do_and(done2);
 
-	 if (! done.isEmpty())
+	 if (! ne2.isEmpty())
 	    last = print(ne, fa->action, EXPORT);
 
 	 done.do_or(*ne);
