@@ -1,48 +1,46 @@
-//  $Id: aoe.cc 1.8 Fri, 31 Oct 1997 18:34:26 -0800 wlee $
-// 
+//  $Id: aoe.cc 1.5.1.9.1.20.1.2 Wed, 07 Apr 1999 09:23:56 -0700 cengiz $
+//
 //  Copyright (c) 1994 by the University of Southern California
-//  and/or the International Business Machines Corporation.
 //  All rights reserved.
 //
-//  Permission to use, copy, modify, and distribute this software and
-//  its documentation in source and binary forms for lawful
-//  non-commercial purposes and without fee is hereby granted, provided
-//  that the above copyright notice appear in all copies and that both
-//  the copyright notice and this permission notice appear in supporting
-//  documentation, and that any documentation, advertising materials,
-//  and other materials related to such distribution and use acknowledge
-//  that the software was developed by the University of Southern
-//  California, Information Sciences Institute and/or the International
-//  Business Machines Corporation.  The name of the USC or IBM may not
-//  be used to endorse or promote products derived from this software
-//  without specific prior written permission.
+//  Permission to use, copy, modify, and distribute this software and its
+//  documentation in source and binary forms for lawful non-commercial
+//  purposes and without fee is hereby granted, provided that the above
+//  copyright notice appear in all copies and that both the copyright
+//  notice and this permission notice appear in supporting documentation,
+//  and that any documentation, advertising materials, and other materials
+//  related to such distribution and use acknowledge that the software was
+//  developed by the University of Southern California, Information
+//  Sciences Institute. The name of the USC may not be used to endorse or
+//  promote products derived from this software without specific prior
+//  written permission.
 //
-//  NEITHER THE UNIVERSITY OF SOUTHERN CALIFORNIA NOR INTERNATIONAL
-//  BUSINESS MACHINES CORPORATION MAKES ANY REPRESENTATIONS ABOUT
-//  THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
-//  PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-//  INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND 
-//  NON-INFRINGEMENT.
+//  THE UNIVERSITY OF SOUTHERN CALIFORNIA DOES NOT MAKE ANY
+//  REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
+//  PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
+//  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
+//  TITLE, AND NON-INFRINGEMENT.
 //
-//  IN NO EVENT SHALL USC, IBM, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
-//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT,
-//  TORT, OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH,
-//  THE USE OR PERFORMANCE OF THIS SOFTWARE.
+//  IN NO EVENT SHALL USC, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
+//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT, TORT,
+//  OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH, THE USE
+//  OR PERFORMANCE OF THIS SOFTWARE.
 //
 //  Questions concerning this software should be directed to 
-//  info-ra@isi.edu.
+//  ratoolset@isi.edu.
 //
-//  Author(s): WeeSan Lee (wlee@isi.edu)
+//  Author(s): WeeSan Lee <wlee@ISI.EDU>
 
 
 #if !defined(IDENT_OFF)
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7) 
 static char ident[] = 
-   "@(#)$Id: aoe.cc 1.8 Fri, 31 Oct 1997 18:34:26 -0800 wlee $";
+   "@(#)$Id: aoe.cc 1.5.1.9.1.20.1.2 Wed, 07 Apr 1999 09:23:56 -0700 cengiz $";
 #endif
 #endif
 
+#include "config.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -52,18 +50,16 @@ static char ident[] =
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "aoe.hh"
-#include "ASMap.h"      // To include AS_map global variable
-#include "Argv.hh"
-#include "version.hh"
-#include "trace.hh"
 #include "aoe.xbm"
-#include "rusage.hh"
-#include "net.hh"       // for class ipAddr
 #include "TclVar.hh"
-
-// Note:
-// 1. AS_Map is a global declared class variable.
-
+#include "util/Argv.hh"
+#include "util/version.hh"
+#include "util/trace.hh"
+#include "util/rusage.hh"
+#include "util/net.hh"       // for class ipAddr
+#include "irr/rawhoisc.hh"
+#include "irr/ripewhoisc.hh"
+#include "irr/birdwhoisc.hh"
 
 //*******************************
 // Some TK variables defined here
@@ -86,24 +82,27 @@ char *tkStatusLine                 = ".statusline.content";
 
 //***********************************
 // Some global variables defined here
-Rusage ru;
-int opt_rusage   = 0;
+bool opt_rusage  = false;
+Rusage ru(clog, &opt_rusage);
 char *opt_my_as  = NULL;
 char *opt_rcfile = NULL;
 char *display    = NULL;
+ASt myAS         = 0;
 
 
 //**********************
 // Define the usage here
 void usage(void)  
 {
-   fprintf(stderr, "\nUsage: aoe  [-help] [-T all | whois_query | whois_response | input]"
-	           "\n            [-D debug_level] [-version]"
-	           "\n            [-h server_ip] [-p server_port] [-s db_order]"
-	           "\n            [-rusage] [-ignore_errors] [-report_errors]"
-	           "\n            [-display  display]"
-	           "\n            [-as as_no]"
-	           "\n\n\a\a"); 
+   fprintf(stderr, 
+	"\nUsage: aoe  [-help] [-T all | whois_query | whois_response | input]"
+	"\n            [-D debug_level] [-version]"
+	"\n            [-h server_ip] [-p server_port] [-s db_order]"
+        "\n            [-protocol [rawhoisd | ripe]]"
+	"\n            [-rusage] [-ignore_errors] [-report_errors]"
+	"\n            [-display  display]"
+	"\n            [-as as_no]"
+	"\n\n\a\a"); 
 }
 
 
@@ -116,7 +115,7 @@ int start_tracing(char *dst, char *key, char *nextArg) {
 }
 
 int start_debugging(char *dst, char *key, char *nextArg) {
-   Debug(if (nextArg) dbg.enable(atoi(argv[i])));
+   Debug(if (nextArg) dbg.enable(atoi(nextArg)));
    return 1; // return 1 to signify nextArg is used by us
 }
 
@@ -124,30 +123,22 @@ void init_and_set_options (int argc, char **argv, char **envp) {
    ArgvInfo argTable[] = {
       // RAToolSet common arguments
       // key, type, src, dst, help
-      {"-T", ARGV_FUNC, (char *) &start_tracing,      (char *) NULL,
+      {"-T", ARGV_FUNC, (char *) &start_tracing,           (char *)NULL,
        "Start tracing the next argument"},
-      {"-D", ARGV_FUNC, (char *) &start_debugging,    (char *) NULL,
+      {"-D", ARGV_FUNC, (char *) &start_debugging,         (char *)NULL,
        "Start debugging the next argument"},
-      {"-version", ARGV_FUNC, (char *) &version,      (char *) NULL,
+      {"-version", ARGV_FUNC, (char *) &version,           (char *)NULL,
        "Show version"},
-      {"-h", ARGV_FUNC, (char *) &Whois::ArgvHost,    (char *) NULL,
-       "Host name of the RAWhoisd server"},
-      {"-p", ARGV_FUNC, (char *) &Whois::ArgvPort,    (char *) NULL,
-       "Port number of the RAWhoisd server"},
-      {"-s", ARGV_FUNC, (char *) &Whois::ArgvSources, (char *) NULL,
-       "Order of databases"},
-      {"-rusage", ARGV_CONSTANT, (char *) 1,          (char *) &opt_rusage,
+      {"-rusage", ARGV_BOOL, (char *)NULL,                 (char *)&opt_rusage,
        "On termination print resource usage"},
-      {"-ignore_errors", ARGV_FUNC, (char *)&Whois::IgnoreErrors, (char *)NULL,
-       "Ignore IRR error and warning messages"},
-      {"-report_errors", ARGV_FUNC, (char *)&Whois::ReportErrors, (char *)NULL,
-       "Print IRR error and warning messages"},
+
+      IRR_COMMAND_LINE_OPTIONS,
 
       // aoe specific arguments
-      {"-as",  ARGV_STRING,    (char *) NULL,    (char *) &opt_my_as,
+      {"-as",  ARGV_STRING,     (char *)NULL, (char *) &opt_my_as,
        "AS number of the aut-num object to use"},
-      {"-rcfile", ARGV_STRING, (char *)NULL, (char *)&opt_rcfile,
-       "aoe resource file (default: ~/.aoerc"},
+      {"-rcfile", ARGV_STRING,  (char *)NULL, (char *)&opt_rcfile,
+       "aoe resource file (default: ~/.aoerc)"},
       {"-display", ARGV_STRING, (char *)NULL, (char *)&display,
        "X display"},
 
@@ -155,20 +146,7 @@ void init_and_set_options (int argc, char **argv, char **envp) {
       {(char *) NULL, ARGV_END, (char *) NULL, (char *) NULL, (char *) NULL}
    };
 
-   for (char **p = envp; *p != NULL; p++) {
-      if (strncmp(*p, "IRR_HOST=", 9) == 0)  {
-         whois.SetDefaultHost(*p + 9);
-         continue;
-      }
-      if (strncmp(*p, "IRR_PORT=", 9) == 0)  {
-         whois.SetDefaultPort(atoi(*p + 9));
-         continue;
-      }
-      if (strncmp(*p, "IRR_SOURCES=", 12) == 0)  {
-         whois.SetDefaultSources(*p + 12);
-         continue;
-      }
-   }
+   IRR::handleEnvironmentVariables(envp);
 
    if (ParseArgv(&argc, argv, argTable, 0) != ARGV_OK) {
       usage();
@@ -180,83 +158,118 @@ void init_and_set_options (int argc, char **argv, char **envp) {
       opt_my_as = argv[1];
       break;
    case 1:
-      if (!opt_my_as) {
-	 ipAddr myip;
-	 myip.setToMyHost();
-	 opt_my_as = AS_map(AS_map.add_entry(myip));
-      }
       break;
    default:
       usage();
       exit(1);
    }
+
+   if (opt_my_as) 
+     myAS = atoi(opt_my_as + 2);
+   else 
+     {
+     IRR *pcIrr;
+     if (!(pcIrr = IRR::newClient())) 
+       {
+       usage();
+       exit(1);
+       }
+
+     ipAddr myip;
+     myip.setToMyHost();
+     
+     if ((myAS = pcIrr->getOrigin("%s/32", myip.getIpaddr())) == 0)
+       {
+       cerr << "aoe could not guess your AS number, "
+	 "please specify an AS number on the command line." 
+	    << endl;
+       delete pcIrr;
+       exit(1);
+       }
+     delete pcIrr;
+   }
 }
 
-
-//**************************
-// Some misc. functions here
-static char *strupr(char *c) 
+/*
+static void colorSyntax(TclText &tt)
 {
-   if (!c) return NULL;
-   for (char *p = c; *p; p++)
-      if (isascii(*p) && islower(*p))
-         *p = toupper(*p);
-   return c;
+  for (int i = 0; i < tt.getTotalLine(); i++)
+    {
+    char pzcBuffer[16];
+    const char *pzcLine = tt.getLine(i);
+    char *p; 
+    if (pzcLine)
+      if ((p = strchr(pzcLine, ':')) != NULL)
+	{
+	int iPos = p - pzcLine;
+	sprintf(pzcBuffer, "%d", iPos);
+	tt.addLineTag(i, "Attribute", "0", pzcBuffer);
+	sprintf(pzcBuffer, "%d", iPos + 1);
+	tt.addLineTag(i, "Value", pzcBuffer, "end");
+	}
+      else
+	tt.addLineTag(i, "Value", "0", "end");
+    }
+//  tt.configureTag("Attribute", "-foreground DeepPink -font *-times-*-i-*"); 
+  tt.configureTag("Attribute", "-foreground DeepPink"); 
+  tt.configureTag("Value", "-foreground blue");
+}
+*/
+
+static TclText &operator<<(TclText &tt, const AutNum &au)
+{
+  tt.clear();
+  tt.insert(((AutNum &) au).contents, ((AutNum &) au).size);
+  tt.removeLastBlankLines();
+  TclApplication::evalf("aoe:colorSyntax %s", tt.getName());
+  return tt;
 }
 
-TclText &operator<<(TclText &tt, AutNum &au)
+/*
+// Got core dump if strstream class is used with compiler option -frtti
+static TclText &operator<<(TclText &tt, const AutNum &au)
 {
-   strstream ss; 
-   //   tt.enable();
+   strstream ss;
    tt.clear();
    ss << au << '\0';       // Always null-terminated!!!
    tt.insert(ss.str());
-   tt.removeLastLine();    // Usually a blank line at the end
-   tt.addLineTag("tag", "0", "14");
-   tt.configureTag("tag", "-foreground DeepPink");
-   tt.addLineTag("content", "14", "end");
-   tt.configureTag("content", "-foreground blue");
    ss.rdbuf()->freeze(0);  // Users have to unfreeze the buffer explicitly
-   //   tt.disable();
+   tt.removeLastBlankLines();
+   TclApplication::evalf("aoe:colorSyntax %s", tt.getName());
    return tt;
 }
+*/
 
 TclText &operator<<(TclText &t1, TclText &t2)
 {
-   const char *p = t2.getAll();
-   if (*p == '\n') return t1;
-   //   t1.enable();
-   //   t1.insert("\n");
+   char *p = strdup(t2.getAll());
+//   if (*p == '\n') return t1;
    t1.insert(p);
-   //   t1.removeLastLine();
-   //   t1.disable();
+   free(p);
    return t1;
 }
 
-TclList &operator<<(TclList &tl, ListHead<ASPeer> &lh)
+TclList &operator<<(TclList &tl, List<ASPeer> &lh)
 {
-   char pzcBuffer[64];
+   char pzcBuffer[64], pzcASPeerNo[16];
    for (ASPeer *pcASPeer = lh.head();
 	pcASPeer;
-	pcASPeer = lh.next(pcASPeer->cNode))
+	pcASPeer = lh.next(pcASPeer))
       {
+      sprintf(pzcASPeerNo, "AS%d", pcASPeer->getNo());
       switch (pcASPeer->getType())
 	 {
 	 case dASPeerFromIRR:
-	    sprintf(pzcBuffer, "{%-8s (IRR)}", 
-		    AS_map(pcASPeer->getNoInPix()));
+	    sprintf(pzcBuffer, "{%-8s (IRR)}", pzcASPeerNo);
 	    break;
 	 case dASPeerFromBGP:
-	    sprintf(pzcBuffer, "{%-8s (BGP)}", 
-		    AS_map(pcASPeer->getNoInPix()));
+	    sprintf(pzcBuffer, "{%-8s (BGP)}", pzcASPeerNo);
 	    break;
 	 case dASPeerFromBoth:
-	    sprintf(pzcBuffer, "{%-8s (IRR & BGP)}", 
-		    AS_map(pcASPeer->getNoInPix()));
+	    sprintf(pzcBuffer, "{%-8s (IRR & BGP)}", pzcASPeerNo);
 	    break;
 	 case dASPeerNew:
-	    sprintf(pzcBuffer, "{%-8s (NEW)}",
-		    AS_map(pcASPeer->getNoInPix()));
+	    sprintf(pzcBuffer, "{%-8s (NEW)}", pzcASPeerNo);
 	    break;
 	 default:
 	    sprintf(pzcBuffer, "{Shoudn't happen!}");
@@ -267,16 +280,16 @@ TclList &operator<<(TclList &tl, ListHead<ASPeer> &lh)
    return tl;
 }
 
-ListHead<ASPeer> &operator<<(ListHead<ASPeer> &lh, BgpASPath &ap)
+List<ASPeer> &operator<<(List<ASPeer> &lh, BgpASPath &ap)
 {
    ASPeer *pcASPeer;
    FirstAndLastAS *pcFirstAndLastAS;
    for (pcFirstAndLastAS = ap.cFirstAndLastASes.head(); 
 	pcFirstAndLastAS; 
-	pcFirstAndLastAS = ap.cFirstAndLastASes.next(pcFirstAndLastAS->cNode))
+	pcFirstAndLastAS = ap.cFirstAndLastASes.next(pcFirstAndLastAS))
       {
-      for (pcASPeer = lh.head(); pcASPeer; pcASPeer = lh.next(pcASPeer->cNode))
-	 if (pcASPeer->getNoInPix() == pcFirstAndLastAS->pvFirst) 
+      for (pcASPeer = lh.head(); pcASPeer; pcASPeer = lh.next(pcASPeer))
+	 if (pcASPeer->getNo() == pcFirstAndLastAS->getFirst()) 
 	    break;
       if (pcASPeer) 
 	 // Found!
@@ -284,8 +297,8 @@ ListHead<ASPeer> &operator<<(ListHead<ASPeer> &lh, BgpASPath &ap)
       else
 	 {
 	 // Not found!
-	 pcASPeer = new ASPeer(pcFirstAndLastAS->pvFirst, dASPeerFromBGP);
-	 lh.append(pcASPeer->cNode);
+	 pcASPeer = new ASPeer(pcFirstAndLastAS->getFirst(), dASPeerFromBGP);
+	 lh.append(pcASPeer);
 	 }
       }
    return lh;
@@ -323,167 +336,175 @@ int ListPeer::updateWindowTitle(TclApplication *pcTcl, int iCategory,
    return TCL_OK;
 }
 
-int ListPeer::displayIRRPolicy(int iImport, int iExport, Pix pvASPeer,
-			       TclText *pcText, AutNum *pcAutNum)
+int ListPeer::displayIRRPolicy(int iImport, int iExport, ASt tAS,
+			       TclText *pcText, const AutNum *pcAutNum)
 {
-   // Clear the text widget first no matter if the peer exist or not!
-  //   pcText->enable();
+  // Clear the text widget first no matter if the peer exist or not!
    pcText->clear();
-
-   ASPolicy *p = pcAutNum->find_peer(pvASPeer);
-   if (p) 
-      {
-      strstream ss;
-      if (iImport) p->printASIn(ss);
-      if (iExport) p->printASOut(ss);
-      ss << '\0';  // This is a must otherwise garbage will appear
-      // Fill the text widget
-      pcText->insert(ss.str());
-      pcText->removeLastLine();
-      ss.rdbuf()->freeze(0);  // This is also a must
-      }
-
-   //   pcText->disable();
-
+   if (pcAutNum)
+     {
+     if (iImport)
+       for (AutNumImportIterator itr(pcAutNum, tAS); itr; itr++)
+	 pcText->insert(itr()->getObject()->contents + itr()->getOffset(), 
+			itr()->getLen());
+     if (iExport)
+       for (AutNumExportIterator itr(pcAutNum, tAS); itr; itr++)
+	 pcText->insert(itr()->getObject()->contents + itr()->getOffset(), 
+			itr()->getLen());
+     pcText->removeLastBlankLines();
+     }
    return 1;
 }
 
 int ListPeer::generatePolicyFromBGP(int iImport, int iExport,
-				    Pix pvAS, Pix pvASPeer, TclText *pcText, 
-				    BgpASPath *pcBgpASPath, int iASPeerType)
+				    ASt tMyAS, ASt tPeerAS, TclText *pcText, 
+				    BgpASPath *pcBgpASPath, int iPeerASType)
 {
    strstream ss;
 
    // Clear the text widget first no matter if the peer exist or not!
-   //   pcText->enable();
    pcText->clear();   
 
-   if (iASPeerType & dASPeerFromBGP)
+   if (iPeerASType & dASPeerFromBGP)
       {
       if (iImport)
 	 {
-	 // as-in
-	    ss << "as-in:        from "
-	       << AS_map(pvASPeer)
-	       << " 10 accept ";
-	    FirstAndLastAS *pcFirstAndLastAS = pcBgpASPath->find(pvASPeer);
-	    for (Pix pix = pcFirstAndLastAS->pvLast.first(); 
-		 pix; 
-		 pcFirstAndLastAS->pvLast.next(pix))
-	       ss << AS_map(pcFirstAndLastAS->pvLast(pix)) 
-		  << " ";
+	 // import
+	    ss << "import: from "
+	       << "AS" 
+	       << tPeerAS
+	       << endl
+	       << "        action pref = 10;"
+	       << endl
+	       << "        accept ";
+	    FirstAndLastAS *pcFirstAndLastAS = pcBgpASPath->find(tPeerAS);
+	    for (int i = pcFirstAndLastAS->getLast().first(); 
+		 i >= 0; 
+		 i = pcFirstAndLastAS->getLast().next(i))
+	       ss << "AS" << i << " ";
 	    ss << endl;
 	 }
       if (iExport)
-	 // as-out
+	 // export
 	 // Temp hack ???
-	 ss << "as-out:       to "
-	    << AS_map(pvASPeer)
-	    << " announce ANY"
+	 ss << "export: to "
+	    << "AS"
+	    << tPeerAS
+	    << endl
+	    << "        announce ANY"
 	    << endl;
       }
 
    // Fill the text widget
    ss << '\0';
    pcText->insert(ss.str());
-   pcText->removeLastLine();
-   //   pcText->disable();
+   pcText->removeLastBlankLines();
    ss.rdbuf()->freeze(0);  // This is also a must
    return 1;
 }
 
 int ListPeer::generatePolicyFromPeer(int iImport, int iExport,
-				     Pix pvAS, Pix pvASPeer, 
-				     TclText *pcText, int iASPeerType)
+				     ASt tMyAS, 
+				     ASt tPeerAS, const AutNum *pcPeerAutNum, 
+				     TclText *pcText, int iPeerASType)
 {
    strstream ss;
 
    // Clear the text widget first no matter if the peer exist or not!
-   //   pcText->enable();
    pcText->clear();   
-
-   //   AutNum *pcAutNumPeer = AS_map.define(pvASPeer);
-   AutNum *pcAutNumPeer = (AutNum *)AS_map.contents(pvASPeer).definition;
-   ASPolicy *pcPolicyPeer = pcAutNumPeer->find_peer(pvAS);
-
-   if (pcPolicyPeer && (iASPeerType & dASPeerFromIRR))
+   
+   if (pcPeerAutNum && (iPeerASType & dASPeerFromIRR))
       {
       if (iImport) 
 	{
-        // Get the as-out of peer
-	for (Filter_Action *fap = pcPolicyPeer->out.head(); 
-	     fap; 
-	     fap = pcPolicyPeer->out.next(fap->falist))
-	  ss << "as-in:        from "
-	     << AS_map(pvASPeer)
-	     << " 10 accept "
-	     << fap->filter 
-	     << endl;
+        // Get the peer's export attribute
+	for (AutNumExportIterator itr(pcPeerAutNum, tMyAS); itr; itr++)
+	  {
+	  PolicyExpr *policy = itr()->policy;
+	  if (typeid(*policy) == typeid(PolicyTerm)) 
+	    {
+	    ListIterator<PolicyFactor> pf(*(PolicyTerm *)policy);
+	    if (pf)
+	      ss << "import: from "
+		 << "AS" << tPeerAS
+		 << endl
+		 << "        action pref = 10;"
+		 << endl
+		 << "        accept "
+		 << *(pf()->filter)
+		 << endl;
+	    }
+	  }
 	}
       if (iExport)
 	{
-        // Get the as-in of peer
-	for (Filter_Action *fap = pcPolicyPeer->in.head(); 
-	     fap; 
-	     fap = pcPolicyPeer->in.next(fap->falist))
-	  ss << "as-out:       to "
-	     << AS_map(pvASPeer)
-	     << " announce "
-	     << fap->filter 
-	     << endl;
+        // Get the peer's export attribute
+	for (AutNumImportIterator itr(pcPeerAutNum, tMyAS); itr; itr++)
+	  {
+	  PolicyExpr *policy = itr()->policy;
+	  if (typeid(*policy) == typeid(PolicyTerm)) 
+	    {
+	    ListIterator<PolicyFactor> pf(*(PolicyTerm *)policy);
+	    if (pf)
+	      ss << "export: to "
+		 << "AS" << tPeerAS
+		 << endl
+		 << "        announce "
+		 << *(pf()->filter)
+		 << endl;
+	    }
+	  }
 	}
       }
 
    // Fill the text widget
    ss << '\0';
    pcText->insert(ss.str());
-   pcText->removeLastLine();
-   //   pcText->disable();
+   pcText->removeLastBlankLines();
    ss.rdbuf()->freeze(0);  // This is also a must
    return 1;
 }
 
 int ListPeer::generateTemplatePolicy(int iCategory, int iImport, int iExport,
-				     Pix pvAS, Pix pvASPeer, TclText *pcText)
+				     char *pzcMyAS, char *pzcPeerAS, 
+				     TclText *pcText)
 {
    TclApplication *pcApp = (TclApplication *)getExtraArgument();
    strstream ss;
    char pzcBuffer[128];
    // Clear the text widget first no matter if the peer exist or not!
-   //   pcText->enable();
    pcText->clear();   
 
    if (iImport)
       {
-      // as-in
-      sprintf(pzcBuffer, "PolicyTemplate(%d,as-in)", iCategory - 4);
-      ss << "as-in:        "
-	 << substitute(pcApp->getVar(pzcBuffer), pvAS, pvASPeer)
+      // import
+      sprintf(pzcBuffer, "PolicyTemplate(%d,import)", iCategory - 4);
+      ss << "import: "
+	 << substitute(pcApp->getVar(pzcBuffer), pzcMyAS, pzcPeerAS)
 	 << endl;
       }
    if (iExport)
       {
-      // as-out
-      sprintf(pzcBuffer, "PolicyTemplate(%d,as-out)", iCategory - 4);
-      ss << "as-out:       "
-	 << substitute(pcApp->getVar(pzcBuffer), pvAS, pvASPeer)
+      // export
+      sprintf(pzcBuffer, "PolicyTemplate(%d,export)", iCategory - 4);
+      ss << "export: "
+	 << substitute(pcApp->getVar(pzcBuffer), pzcMyAS, pzcPeerAS)
 	 << endl;
       }
    // Fill the text widget
    ss << '\0';
    pcText->insert(ss.str());
-   pcText->removeLastLine();
-   //   pcText->disable();
+   pcText->removeLastBlankLines();
    ss.rdbuf()->freeze(0);  // This is also a must
    return 1;
 }
 
-char *ListPeer::substitute(char *pzcString, Pix pvAS, Pix pvASPeer)
+char *ListPeer::substitute(char *pzcString, char *pzcMyAS, char *pzcPeerAS)
 {
    static char pzcBuffer[dBufferLength + 1 + 32];
-   char *pzcMyAS = "$MyAS", *pzcPeerAS = "$PeerAS";
+   char *pzcMyASVar = "$MyAS", *pzcPeerASVar = "$PeerAS";
    char *p1, *p2;
-   int iMyASLen = strlen(pzcMyAS), iPeerASLen = strlen(pzcPeerAS);
+   int iMyASVarLen = strlen(pzcMyASVar), iPeerASVarLen = strlen(pzcPeerASVar);
    int iTotal = 0;
    p1 = p2 = pzcString;
    pzcBuffer[0] = 0;
@@ -492,19 +513,19 @@ char *ListPeer::substitute(char *pzcString, Pix pvAS, Pix pvASPeer)
       int iLen = p2 - p1;
       strncpy(pzcBuffer + iTotal, p1, iLen);
       iTotal += iLen;
-      if (strncmp(p2, pzcMyAS, iMyASLen) == 0)
+      if (strncmp(p2, pzcMyASVar, iMyASVarLen) == 0)
 	 {
-	 p2 += iMyASLen;
-	 char *p = AS_map(pvAS);
+	 p2 += iMyASVarLen;
+	 char *p = pzcMyAS;
 	 iLen = strlen(p);
 	 strncpy(pzcBuffer + iTotal, p, iLen);
 	 iTotal += iLen;
 	 }
       else
-	if (strncmp(p2, pzcPeerAS, iPeerASLen) == 0)
+	if (strncmp(p2, pzcPeerASVar, iPeerASVarLen) == 0)
 	   {
-	   p2 += iPeerASLen;
-	   char *p = AS_map(pvASPeer);
+	   p2 += iPeerASVarLen;
+	   char *p = pzcPeerAS;
 	   iLen = strlen(p);
 	   strncpy(pzcBuffer + iTotal, p, iLen);
 	   iTotal += iLen;
@@ -548,56 +569,52 @@ int ListPeer::command(int argc, char *argv[])
    int iImport      = pcApp->pcImportButton->getOption();
    int iExport      = pcApp->pcExportButton->getOption();
    int iCategory    = pcApp->pcCategoryButton->getOption();
-   int iASPeerIndex = atoi(argv[1]);
-   int iASPeerType  = pcApp->getASPeerType(iASPeerIndex);
-   Pix pvAS         = pcApp->getASNoInPix();
-   Pix pvASPeer     = pcApp->getASPeerNoInPix(iASPeerIndex);
+   int iPeerASIndex = atoi(argv[1]);
+   int iPeerASType  = pcApp->getASPeerType(iPeerASIndex);
+
+   ASt tMyAS        = pcApp->getASNo();
+   ASt tPeerAS      = pcApp->getASPeerNo(iPeerASIndex);
+   
+   char pzcMyAS[16], pzcPeerAS[16];
+   sprintf(pzcMyAS, "AS%d", tMyAS);
+   sprintf(pzcPeerAS, "AS%d", tPeerAS);
 
    // Setup $PeerAS properly
-   if (!pcApp->evalf("set PeerAS %s", AS_map(pvASPeer))) return TCL_ERROR;
+   if (!pcApp->evalf("set PeerAS %s", pzcPeerAS)) return TCL_ERROR;
 
    // Make the window title more meaningful
-   updateWindowTitle(pcApp, iCategory, pcApp->getASNoInString());
-   if (!pcApp->pcNeighborLabel->attach(AS_map(pvASPeer))) return TCL_ERROR;
-   pcApp->pcPolicyShowButton->labelBothString(AS_map(pvASPeer));
+   updateWindowTitle(pcApp, iCategory, pzcMyAS);
+   if (!pcApp->pcNeighborLabel->attach(pzcPeerAS)) return TCL_ERROR;
+   pcApp->pcPolicyShowButton->labelBothString(pzcPeerAS);
    if (!pcApp->eval("showPolicyText")) return TCL_ERROR;
    
 
-   // Try to define peer first.  It's fine to do it everytime 
-   // since it will be cached in the memory and won't be duplicated
-   AutNum *pcAutNumPeer = AS_map.define(pvASPeer);
+   const AutNum *pcPeerAutNum = pcApp->pcIrr->getAutNum(tPeerAS);
    // Display peer of working AS
-   displayIRRPolicy(iExport, iImport, pcApp->getASNoInPix(), 
-			  pcApp->pcNeighborText, pcAutNumPeer);
+   displayIRRPolicy(iExport, iImport, tMyAS, 
+		    pcApp->pcNeighborText, pcPeerAutNum);
 
    switch (iCategory)
       {
       case 0: // from IRR Database
 	 {
 	 // Display working AS
-	 displayIRRPolicy(iImport, iExport, pvASPeer, 
+	 displayIRRPolicy(iImport, iExport, tPeerAS, 
 			  pcApp->pcDefText, pcApp->pcAutNum);
-/*
-	 // Try to define peer first.  It's fine to do it everytime 
-	 // since it will be cached in the memory and won't be duplicated
-	 AutNum *pcAutNumPeer = AS_map.define(pvASPeer);
-
-	 // Display peer of working AS
-	 displayIRRPolicy(iExport, iImport, pcApp->getASNoInPix(), 
-			  pcApp->pcNeighborText, pcAutNumPeer);
-*/
 	 break;
 	 }
       case 1: // from BGP Dump (AS-Path)
 //	 pcApp->pcNeighborText->clear();
 	 generatePolicyFromBGP(iImport, iExport, 
-			       pvAS, pvASPeer, pcApp->pcDefText, 
-			       &pcApp->cBgpASPath, iASPeerType);
+			       tMyAS, tPeerAS, pcApp->pcDefText, 
+			       &pcApp->cBgpASPath, iPeerASType);
 	 break;
       case 2: // copy from peer (valid only for AS from IRR)
 	 generatePolicyFromPeer(iImport, iExport, 
-				pvAS, pvASPeer,	
-				pcApp->pcDefText, iASPeerType);
+				tMyAS, 
+				tPeerAS,
+				pcPeerAutNum,	
+				pcApp->pcDefText, iPeerASType);
 	 break;
       case 3: // Shouldn't happen here since it's just a separator!
 	 break;
@@ -605,10 +622,17 @@ int ListPeer::command(int argc, char *argv[])
 	 {
 //	 pcApp->pcNeighborText->clear();
 	 generateTemplatePolicy(iCategory, iImport, iExport, 
-				pvAS, pvASPeer,	pcApp->pcDefText);
+				pzcMyAS, pzcPeerAS, pcApp->pcDefText);
 	 break;
 	 }
       }
+
+   // Color syntaxing before leaving
+   if (!pcApp->evalf("aoe:colorSyntax %s", pcApp->pcDefText->getName()))
+     return TCL_ERROR;
+   if (!pcApp->evalf("aoe:colorSyntax %s", pcApp->pcNeighborText->getName()))
+     return TCL_ERROR;
+
    return TCL_OK;
 }
 
@@ -619,10 +643,10 @@ int LoadBgpDump::command(int argc, char *argv[])
    AoeApplication *pcApp = (AoeApplication *)getExtraArgument();
    // Call lexer 
    extern int loadBgpDump(BgpASPath &);  // Declared in bgp_dump.l
-   extern FILE *bgpin;
+   extern FILE *bgp_dumpin;
    FILE *fpIn = fopen(argv[1], "r");
    if (!fpIn) return TCL_ERROR;
-   bgpin = fpIn;
+   bgp_dumpin = fpIn;
    loadBgpDump(pcApp->cBgpASPath);
    fclose(fpIn);
    // Duplicate first to last if last is empty
@@ -671,11 +695,11 @@ int UpdatePolicy::replace(int iTarget)
    int i;
    for (i = 0, pcASPeer = pcApp->cASPeers.head(); 
 	i < iTarget && pcASPeer; 
-	i++, pcASPeer = pcApp->cASPeers.next(pcASPeer->cNode)) ;
+	i++, pcASPeer = pcApp->cASPeers.next(pcASPeer)) ;
    if (pcASPeer)
      {
-     Pix pvASPeer = pcASPeer->getNoInPix();   
-     pcApp->pcAutNum->remove_peer(pvASPeer);
+     ASt tPeerAS = pcASPeer->getNo();   
+     pcApp->pcAutNum->removePeer(tPeerAS);
      *pcApp->pcPolicyText << *pcApp->pcAutNum; // Redisplay
      }
    return append();
@@ -720,9 +744,11 @@ int ShowPolicyText::command(int argc, char *argv[])
 	 int iActiveItem = pcApp->pcASPeerList->getCurrentItem();
 	 if (iActiveItem >= 0)
 	    {
-	    Pix pvASPeer = pcApp->getASPeerNoInPix(iActiveItem);
+	    const AutNum *pcPeerAutNum = 
+	      pcApp->pcIrr->getAutNum(pcApp->getASPeerNo(iActiveItem));
 	    pcApp->pcPolicyText->clear();
-	    *pcApp->pcPolicyText << *(AS_map.define(pvASPeer));
+	    // Might be NULL here!
+	    if (pcPeerAutNum) *pcApp->pcPolicyText << *pcPeerAutNum;
 	    // Disable improper buttons
 	    //	    pcApp->pcPolicyDeleteButton->disable();
 	    pcApp->pcPolicyEditButton->disable();
@@ -760,34 +786,30 @@ int FileOpen::command(int argc, char *argv[])
    if (argc != 2) return TCL_ERROR;
    AoeApplication *pcApp = (AoeApplication *)getExtraArgument();
    FILE *fp;
+   int iResult = TCL_OK;
    struct stat sFileStat;
-   // Delete original one
-   if (pcApp->pcAutNum == AS_map.contents(pcApp->getASNoInPix()).definition)
-      delete pcApp->pcAutNum;
    // Get size of the file
    if (stat(argv[1], &sFileStat)) return TCL_ERROR;
    // Open the file for reading
    if ((fp = fopen(argv[1], "r")) == NULL) return TCL_ERROR;
-   // Simulate WhoisParser::ParseObject()
-   extern AutNum *irr_parser_as;
-   extern int irr_lexer_input_size; 
-   extern FILE *irrin;
-   extern int irrparse();
-   irr_parser_as = pcApp->pcAutNum = new AutNum;
-   AS_map.contents(pcApp->getASNoInPix()).definition = (void *)irr_parser_as;
-   irr_lexer_input_size = sFileStat.st_size; 
-   irrin = fp;
-   irrparse();
-
-   AS_map.handle_extended_and_original_attr(pcApp->pcAutNum);
-   
+   char *pzcBuffer = new char [sFileStat.st_size + 1];
+   // Read the whole thing
+   if (fread(pzcBuffer, 1, sFileStat.st_size, fp) != sFileStat.st_size)
+     iResult = TCL_ERROR;
    fclose(fp);
-
-   // Refill policy text widget, if the show button is inappropriate, toggle it
-   if (pcApp->pcPolicyShowButton->getOption()) 
-      pcApp->pcPolicyShowButton->toggle();
-   if (!pcApp->eval("showPolicyText")) return TCL_ERROR;
-   return TCL_OK;
+   pzcBuffer[sFileStat.st_size] = 0;
+   if (iResult == TCL_OK)
+     {
+     // Delete original one
+     if (pcApp->pcAutNum) delete pcApp->pcAutNum;
+     Buffer b(pzcBuffer, sFileStat.st_size);
+     if ((pcApp->pcAutNum = new AutNum(b)) == NULL)
+       iResult = TCL_ERROR;
+     else
+       // Refill policy text widget
+       *pcApp->pcPolicyText << *pcApp->pcAutNum;
+     }
+   return iResult;
 }
 
 int FileSave::command(int argc, char *argv[])
@@ -805,13 +827,8 @@ int FileRevert::command(int argc, char *argv[])
 {
    if (argc != 1) return TCL_ERROR;
    AoeApplication *pcApp = (AoeApplication *)getExtraArgument();
-   // Delete original one
-   if (pcApp->pcAutNum == AS_map.contents(pcApp->getASNoInPix()).definition)
-      {
-      delete pcApp->pcAutNum;
-      pcApp->pcAutNum = NULL;
-      AS_map.contents(pcApp->getASNoInPix()).definition = NULL;
-      }
+   // Delete original one first
+   if (pcApp->pcAutNum) delete pcApp->pcAutNum;
    // Clear the peer list first
    pcApp->cASPeers.clear();       // Clear the list from memory
    pcApp->pcASPeerList->clear();  // Clear the list from Tk
@@ -853,9 +870,9 @@ int AddPeer::command(int argc, char *argv[])
    if (argc != 2) return TCL_ERROR;
    AoeApplication *pcApp = (AoeApplication *)getExtraArgument();
    // Allocate a new peer
-   ASPeer *pcASPeer = new ASPeer(AS_map.add_entry(argv[1]), dASPeerNew);
+   ASPeer *pcASPeer = new ASPeer(atoi(argv[1] + 2), dASPeerNew);
    if (!pcASPeer) return TCL_ERROR;
-   pcApp->cASPeers.append(pcASPeer->cNode);
+   pcApp->cASPeers.append(pcASPeer);
    // Clear the aspeer list
    if (!pcApp->pcASPeerList->clear()) TCL_ERROR;
    // Refill the AS Peer list
@@ -883,16 +900,18 @@ int DeletePeer::command(int argc, char *argv[])
 	 }
       return TCL_OK;
       }
-   Pix pvASPeer = NULL;
    ASPeer *pcASPeer;
    int i, iTarget = atoi(argv[1]);
    for (i = 0, pcASPeer = pcApp->cASPeers.head(); 
 	i < iTarget && pcASPeer; 
-	i++, pcASPeer = pcApp->cASPeers.next(pcASPeer->cNode)) ;
+	i++, pcASPeer = pcApp->cASPeers.next(pcASPeer)) ; //For loop ends here
    if (pcASPeer)
      {
-     pvASPeer = pcASPeer->getNoInPix();
-     pcASPeer->cNode.un_link();
+     pcApp->pcAutNum->removePeer(pcASPeer->getNo());
+     // Redisplay the contain of AutNum in policy text widget
+     *pcApp->pcPolicyText << *pcApp->pcAutNum;
+     // Remove the peer AS from the list
+     pcApp->cASPeers.remove(pcASPeer);
      delete pcASPeer;
      // Clear the aspeer list
      if (!pcApp->pcASPeerList->clear()) TCL_ERROR;
@@ -904,12 +923,6 @@ int DeletePeer::command(int argc, char *argv[])
      if (!pcApp->pcNeighborLabel->attach("Empty")) return TCL_ERROR;
      pcApp->pcPolicyShowButton->labelBothString("Empty");
      }
-   // else should happen here
-
-   if (pvASPeer)
-      if (pcApp->pcAutNum->remove_peer(pvASPeer))
-	 // Redisplay the contain of AutNum in policy text widget
-	 *pcApp->pcPolicyText << *pcApp->pcAutNum;
 
    // Take care of show policy button
    if (pcApp->pcPolicyShowButton->getOption())
@@ -917,7 +930,6 @@ int DeletePeer::command(int argc, char *argv[])
      pcApp->pcPolicyShowButton->toggle();
      if (!pcApp->eval("showPolicyText")) return TCL_ERROR;
      }
-
    return TCL_OK;
 }
 
@@ -949,12 +961,16 @@ int CreatePolicyTemplateMenu::command(int argc, char *argv[])
 //******************************
 // Application class starts here
 
+#include "gnug++/SetOfUInt.hh"
+
 AoeApplication::AoeApplication(char *pzcAppName, 
 			       char *pzcDisplay,
-			       char *pzcRCFile,
-			       char *pzcASNo) :
-   TclApplication(pzcAppName, pzcDisplay, pzcRCFile),
-   pzcASNo(pzcASNo),
+			       char *pzcDefaultResource,
+			       char *pzcResourceFile,
+			       ASt tASNo) :
+   TclApplication(pzcAppName, pzcDisplay, pzcDefaultResource, pzcResourceFile),
+   tASNo(tASNo),
+   pcAutNum(NULL),
    pcASPeerList(NULL),
    pcPolicyText(NULL),
    pcDefText(NULL),
@@ -970,14 +986,21 @@ AoeApplication::AoeApplication(char *pzcAppName,
    pcPolicyShowButton(NULL),
    pcStatusLine(NULL)
 {
+  sprintf(pzcASNo, "AS%d", tASNo);
+  if (!(pcIrr = IRR::newClient()))
+    {
+    usage();
+    exit(1);
+    }
+//  pcIrr->setFastResponseFlag(false);
 }
 
 AoeApplication::~AoeApplication(void)
 {
    // Save settings into resource file before program ends
-   evalf("aoe:saveResource %s", getResource());
+   evalf("aoe:saveResource {%s} {%s}", getResourceFile(), ProjectVersion);
    // Free allocate memory
-   if (!cASPeers.is_empty()) cASPeers.clear();
+   if (!cASPeers.isEmpty()) cASPeers.clear();
    if (pcASPeerList) delete pcASPeerList;
    if (pcPolicyText) delete pcPolicyText;
    if (pcDefText) delete pcDefText;
@@ -992,29 +1015,23 @@ AoeApplication::~AoeApplication(void)
    if (pcPolicyEditButton) delete pcPolicyEditButton;
    if (pcPolicyShowButton) delete pcPolicyShowButton;
    if (pcStatusLine) delete pcStatusLine;
+   if (pcAutNum) delete pcAutNum;
+   if (pcIrr) delete pcIrr;
 }
 
 int AoeApplication::getASInfoFromServer(void)
 {
-   whois.QueryKillResponse("!uF=0");
+   const AutNum *pcAutNumTemp = pcIrr->getAutNum(tASNo);
+   if (!pcAutNumTemp) return 0;
 
-   // Create an AS entry in the AS map
-   pvASNo = AS_map.add_entry(pzcASNo);
-   pcAutNum = AS_map.define(pvASNo);
+   // Get a fresh copy of aut-num object
+   pcAutNum = new AutNum(*pcAutNumTemp);
 
-   if (!pcAutNum->peers.is_empty())
-      {
-      // Get all peers' pix
-      ASPolicy *p;
-      int i;
-      for (p = pcAutNum->peers.head(), i = 0;
-	   p; 
-	   p = pcAutNum->peers.next(p->peers), i++)
-	 {
-	 ASPeer *pcASPeer = new ASPeer(p->peeras, dASPeerFromIRR);
-	 cASPeers.append(pcASPeer->cNode);
-	 }
-      }
+   // Get Peer AS from import/export/default attributes
+   for (ASt tPeerAS = pcAutNum->firstPeerAS(); 
+	tPeerAS != INVALID_AS;
+	tPeerAS = pcAutNum->nextPeerAS())
+     cASPeers.append(new ASPeer(tPeerAS, dASPeerFromIRR));
 
    // Fill out the AS Peer list
    *pcASPeerList << cASPeers;
@@ -1032,17 +1049,17 @@ int AoeApplication::getASInfoFromServer(void)
    return 1;
 }
 
-Pix AoeApplication::getASPeerNoInPix(int i)
+ASt AoeApplication::getASPeerNo(int i)
 {
    ASPeer *pcASPeer;
    int iIndex;
    for (pcASPeer = cASPeers.head(), iIndex = 0;
 	pcASPeer;
-	pcASPeer = cASPeers.next(pcASPeer->cNode), iIndex++)
+	pcASPeer = cASPeers.next(pcASPeer), iIndex++)
       if (iIndex == i) break;
-   if (pcASPeer) return pcASPeer->getNoInPix();
+   if (pcASPeer) return pcASPeer->getNo();
    // Shouldn't come to here
-   return NULL;
+   return 0;
 }
 
 int AoeApplication::getASPeerType(int i)
@@ -1051,7 +1068,7 @@ int AoeApplication::getASPeerType(int i)
    int iIndex;
    for (pcASPeer = cASPeers.head(), iIndex = 0;
 	pcASPeer;
-	pcASPeer = cASPeers.next(pcASPeer->cNode), iIndex++)
+	pcASPeer = cASPeers.next(pcASPeer), iIndex++)
       if (iIndex == i) break;
    if (pcASPeer) return pcASPeer->getType();
    // Shouldn't come to here
@@ -1071,43 +1088,12 @@ int AoeApplication::bootStrap(void)
 
 int AoeApplication::reparseWorkingAutNumArea(void)
 {
-   FILE *fp;
-   struct stat sFileStat;
-   char *pzcTmpFile = getVar("tmpFile");
-   // Write policy to a file
-   if ((fp = fopen(pzcTmpFile, "w")) == NULL)
-      return 0;
-   fputs(pcPolicyText->getAll(), fp);
-   fclose(fp);
-   // Get size of the file
-   if (stat(pzcTmpFile, &sFileStat)) 
-      return 0;
-   // Parse policy from the file
-   if ((fp = fopen(pzcTmpFile, "r")) == NULL)
-      return 0;
-   
-   // For safety check!
-   if (pcAutNum == AS_map.contents(getASNoInPix()).definition)
-      delete pcAutNum;
-
-   // Simulate WhoisParser::ParseObject()
-   extern AutNum *irr_parser_as;
-   extern int irr_lexer_input_size; 
-   extern FILE *irrin;
-   extern int irrparse();
-   irr_parser_as = pcAutNum = new AutNum;
-   AS_map.contents(getASNoInPix()).definition = (void *)irr_parser_as;
-   irr_lexer_input_size = sFileStat.st_size; 
-   irrin = fp;
-   irrparse();
-
-   AS_map.handle_extended_and_original_attr(pcAutNum);
-   fclose(fp);
-   unlink(pzcTmpFile);
-
+   if (pcAutNum) delete pcAutNum;
+   char *pzcPolicyText = strdup(pcPolicyText->getAll());
+   Buffer b(pzcPolicyText, strlen(pzcPolicyText));
+   pcAutNum = new AutNum(b);
    // Refill policy text widget
    *pcPolicyText << *pcAutNum;
-
    return 1;
 }
 
@@ -1119,15 +1105,6 @@ int AoeApplication::init(void)
 
    // eval() is only working after Tcl_Init()
 
-   // Check if the resource file (~/.aoerc) exist or not
-   if (!evalf("file exists %s", getResource())) return 0;
-   if (atoi(getResult()) == 0)
-      {
-      // If not, use the default resource
-      extern char *aoerc_tk;
-      if (!eval(aoerc_tk)) return 0;
-      }
-
    // Load the file select module from fileselect.tk
    extern char *fileselect_tk;
    if (!eval(fileselect_tk)) return 0;
@@ -1202,7 +1179,7 @@ int AoeApplication::init(void)
    if (!insert(pcCommitButton)) return 0;
 
    pcPolicyEditButton = new TclToggleButton(tkPolicyEditButton,
-					    "Edit", "Edit done!",
+					    "Edit", "EDIT",
 					    cbUnCheck);
    if (!insert(pcPolicyEditButton)) return 0;
 
@@ -1232,18 +1209,18 @@ int AoeApplication::run(void)
       {
       fprintf(stderr, "%s", getResult());
       exit(1);
-      }    
+      }
+   return 1;
 }
 
 // Main program
 int main(int argc, char *argv[], char *envp[])
 {
+   schema.initialize();
    init_and_set_options(argc, argv, envp);
    if (!opt_rcfile) opt_rcfile = "~/.aoerc";
-   AoeApplication cAoeApplication("aoe", display, opt_rcfile, 
-				  strupr(opt_my_as));
+   extern char *aoerc_tk;
+   AoeApplication cAoeApplication("aoe", display, aoerc_tk, opt_rcfile, myAS);
    cAoeApplication.run();
-   if (opt_rusage) clog << ru;
    return 0;
 }
-
