@@ -1,4 +1,4 @@
-//  $Id: ripewhoisc.cc 1.4.1.1.1.9.1.11.1.17 Mon, 24 May 1999 17:18:04 -0700 cengiz $
+//  $Id: ripewhoisc.cc 1.4.1.1.1.9.1.11.1.19 Thu, 06 Sep 2001 08:44:03 -0700 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -94,6 +94,20 @@ void RipeWhoisClient::Close()
   write("\n", 1);
 }
 
+static int lastiNoofTry;
+static char *lastBuf;
+static int lastLen;
+
+void RipeWhoisClient::recoverFromSleepingServerBug()
+{
+   cerr << "Warning: server disappeared, re-establishing the connection..." 
+	<< endl;
+   newSock();
+   Open();
+   write(lastiNoofTry, lastBuf, lastLen);
+}
+
+
 // 1. Have to do the buffer myself 
 // 2. Have to look for \n\n\n for the end of each respond 
 //    (might have multiple responds)
@@ -114,10 +128,18 @@ char *RipeWhoisClient::read(void)
       {
       iByteRead = Socket::read(pzcBuffer, PROBE_BUFFER_SIZE);
       if (iByteRead < 0)
-        if (++iNoofTry > NO_OF_TRIAL_TRY)
-          return NULL;
-	else
-	  continue;
+	 if (++iNoofTry > NO_OF_TRIAL_TRY)
+	    return NULL;
+	 else
+	    continue;
+      else if (iByteRead == 0)
+	 if (++iNoofTry > NO_OF_TRIAL_TRY) {
+	    recoverFromSleepingServerBug();
+	    cProbeBuffer.clear();
+	    iNoofTry = 0;
+	    continue;
+	 } else
+	    continue;
       cProbeBuffer.append(pzcBuffer, iByteRead);
       if ((ulSearchResult = cProbeBuffer.find(pzcEor))<= cProbeBuffer.length())
 	break;
@@ -140,6 +162,10 @@ char *RipeWhoisClient::read(void)
 // Modified from Rusty's Socket class
 int RipeWhoisClient::write(int iNoofTry, char *buf, int len)
 {
+   lastiNoofTry = iNoofTry;
+   lastBuf = buf;
+   lastLen = len;
+
     char *b = (char *) buf;
     int l = len;
     while (l > 0) {
@@ -156,7 +182,7 @@ int RipeWhoisClient::write(int iNoofTry, char *buf, int len)
 	// Restore it back to blocking
 	::fcntl(sock, F_SETFL, iFlags);
 	if (iRead == 1) cProbeBuffer.append(pzcOneByte);
-	if (iRead == 0 && errno == 0)
+	if (iRead == 0 && errno != EAGAIN)
 	  {
 	  newSock();
 	  Open();
@@ -366,7 +392,7 @@ bool RipeWhoisClient::getIndirectMembers(const Object *o,
       else
 	 len = strlen(p1) - 1;
       Buffer b(strdup(p1+1), len);
-      Object *mbr = new Object(b);
+      Object *mbr = new Object(b, true);
       if (isAny || isIndirectMember(mbr, mbrs_by_ref))
 	 (*collect)(collectArg, mbr);
       delete mbr;
