@@ -1,4 +1,4 @@
-//  $Id: RadixSet.cc 1.21 Thu, 14 Jan 1999 14:56:48 -0800 cengiz $
+//  $Id: RadixSet.cc 1.23 Tue, 20 Feb 2001 08:11:55 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -123,6 +123,50 @@ bool RadixSet::Iterator::next(u_int &addr, u_int &leng, u_int64_t &rngs) {
    return true;
 }
 
+bool RadixSet::SortedIterator::first(u_int &_addr, u_int &_leng, u_int64_t &_rngs) {
+   RadixSet::Iterator itr(set);
+   PrefixLNode *p;
+   u_int addr, leng;
+   u_int64_t rngs;
+
+   l.clear();
+
+   for (bool ok = itr.first(addr, leng, rngs);
+	ok;
+	ok = itr.next(addr, leng, rngs)) {
+      if (!rngs)
+	 continue;
+      // this is insertion sorting
+      // it can be as bad as O(n^2)
+      // but since radix tree is almost sorted, it will be around (O(n))
+      for (p = l.head(); 
+	   p && (p->addr < addr || (p->addr == addr && p->leng < leng)); 
+	   p = l.next(p))
+	 ;
+      if (p)
+	 l.insertBefore(p, (new PrefixLNode(addr, leng, rngs)));
+      else
+	 l.append((new PrefixLNode(addr, leng, rngs)));
+   }
+
+   return next(_addr, _leng, _rngs);
+}
+
+bool RadixSet::SortedIterator::next(u_int &_addr, u_int &_leng, u_int64_t &_rngs) {
+   PrefixLNode *p = l.head();
+   
+   if (!p)
+      return false;
+   
+   _addr = p->addr;
+   _leng  = p->leng;
+   _rngs = p->rngs;
+   l.remove(p);
+   delete p;
+
+   return true;
+}
+
 bool RadixSet::PrefixIterator::first(u_int &_addr, u_int &_leng) {
 
    for (current = itr.first(); 
@@ -600,7 +644,7 @@ RadixTree *RadixTree::removeRange(u_int64_t _rngs) {
    return this;
 }
 
-RadixTree *RadixTree::and(const RadixTree *b) {
+RadixTree *RadixTree::and_(const RadixTree *b) {
    if (this == b)
       return this;
 
@@ -704,7 +748,7 @@ RadixTree *RadixTree::setminus(const RadixTree *b) {
    return result;
 }
 
-RadixTree *RadixTree::or(const RadixTree *b) {
+RadixTree *RadixTree::or_(const RadixTree *b) {
    if (this == b)
       return this;
 
