@@ -1,4 +1,4 @@
-//  $Id: SetOfPrefix.cc 1.12 Tue, 17 Nov 1998 16:29:25 -0800 wlee $
+//  $Id: SetOfPrefix.cc 1.13 Tue, 20 Feb 2001 08:11:55 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -56,7 +56,7 @@ void SetOfPrefix::insert(const PrefixRanges& b) {
       for (i = b[j].get_n(); i <= b[j].get_m(); ++i)
 	 rngs |= bits[i];
 */
-      if (not)
+      if (not_)
 	 members.remove(b[j].get_ipaddr(), b[j].get_length(), rngs); 
       else
 	 members.insert(b[j].get_ipaddr(), b[j].get_length(), rngs); 
@@ -66,7 +66,7 @@ void SetOfPrefix::insert(const PrefixRanges& b) {
 void SetOfPrefix::remove(const PrefixRanges& b) { 
    if (_universal) {
       _universal = 0;
-      not = 1;
+      not_ = 1;
    }
 
    u_int64_t rngs; int i;
@@ -80,7 +80,7 @@ void SetOfPrefix::remove(const PrefixRanges& b) {
       for (i = b[j].get_n(); i <= b[j].get_m(); ++i)
 	 rngs |= bits[i];
 */
-      if (not)
+      if (not_)
 	 members.insert(b[j].get_ipaddr(), b[j].get_length(), rngs); 
       else
 	 members.remove(b[j].get_ipaddr(), b[j].get_length(), rngs); 
@@ -100,7 +100,7 @@ ostream& operator<<(ostream& stream, SetOfPrefix& set) {
    if (set._universal)
       stream << "UNIVERSAL";
    else {
-      if (set.not)
+      if (set.not_)
 	 stream << "NOT";
 
       stream << set.members;
@@ -119,9 +119,9 @@ void SetOfPrefix::operator |= (const SetOfPrefix& b) { // union
       if (b._universal) {
 	 members.clear();
 	 _universal = 1;
-	 not = 0;
+	 not_ = 0;
       } else {
-	 switch (not*2+b.not) {
+	 switch (not_*2+b.not_) {
 	 case 0: // not = 0, b.not = 0 => this UNION b = members UNION b.members
 	    members |= b.members;
 	    break;
@@ -129,7 +129,7 @@ void SetOfPrefix::operator |= (const SetOfPrefix& b) { // union
 	    c = members;
 	    members = b.members;
 	    members -= c;
-	    not = 1;
+	    not_ = 1;
  	    break;
 	 case 2: // not = 1, b.not = 0 => this UNION b = b.members MINUS members
 	    members -= b.members;
@@ -140,9 +140,9 @@ void SetOfPrefix::operator |= (const SetOfPrefix& b) { // union
 	 default:
 	    assert(0);
 	 }
-	 if (isEmpty() && not) { // case 1,2,3 can cause this
+	 if (isEmpty() && not_) { // case 1,2,3 can cause this
 	    _universal = 1;
-	    not = 0;
+	    not_ = 0;
 	 }
       }
 }
@@ -156,14 +156,14 @@ void SetOfPrefix::operator &= (const SetOfPrefix& b) { // intersection
       if (b.isEmpty()) {
 	 members.clear();
 	 _universal = 0;
-	 not = 0;
+	 not_ = 0;
       } else 
 	 if (_universal) {
 	    _universal = b._universal;
-	    not = b.not;
+	    not_ = b.not_;
 	    members = b.members;
 	 } else {
-	    switch (not*2+b.not) {
+	    switch (not_*2+b.not_) {
 	    case 0: // not = 0, b.not = 0 => this INTSCTN b = members INTSCTN b.members
 	       members &= b.members;
 	       break;
@@ -174,7 +174,7 @@ void SetOfPrefix::operator &= (const SetOfPrefix& b) { // intersection
 	       c = members;
 	       members = b.members;
 	       members -= c;	    
-	       not = 0;
+	       not_ = 0;
 	       break;
 	    case 3: // not = 1, b.not = 1 => this INTSCTN b = members UNION b.members
 	       members |= b.members;
@@ -191,21 +191,21 @@ void SetOfPrefix::operator ~  () { // complement
       _universal = 0;
    } else 
       if (isEmpty()) {
-	 assert(! not);
+	 assert(! not_);
 	 _universal = 1;
       } else
-	 not = ! not;
+	 not_ = ! not_;
 }
 
 int  SetOfPrefix::operator == (const SetOfPrefix& other) const { // equivalance
    return (_universal == other._universal 
-	   && not == other.not
+	   && not_ == other.not_
 	   && members == other.members);
 }
 
 void  SetOfPrefix::operator = (const SetOfPrefix& other) { // assignment
    members.clear();
-   not = other.not;
+   not_ = other.not_;
    _universal = other._universal;
    members = other.members;
 }
