@@ -1,4 +1,4 @@
-//  $Id: peval.cc 1.26.1.9 Mon, 08 Feb 1999 11:14:17 -0800 cengiz $
+//  $Id: peval.cc 1.26.1.11 Tue, 20 Feb 2001 08:11:55 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -97,7 +97,7 @@ void evaluate() {
    AttrIterator<AttrFilter> itr(o, "peval");
 
    NormalExpression *ne = 
-      NormalExpression::evaluate(itr()->filter, -1, opt_expand);
+      NormalExpression::evaluate(itr()->filter, ~0, opt_expand);
    if (ne)
       cout << *ne << endl;
    else
@@ -147,8 +147,6 @@ int expression(char *dst, char *key, char *nextArg) {
    if (!nextArg) return 0;
 
    strcpy(filter, nextArg);
-   evaluate();
-   exit(0);
    return 1;
 }
 
@@ -205,8 +203,6 @@ void init_and_set_options (int argc, char **argv, char **envp) {
    switch (argc) {
    case 2:
       strcpy(filter, argv[1]);
-      evaluate();
-      exit(0);
       break;
    case 1:
       break;
@@ -215,6 +211,11 @@ void init_and_set_options (int argc, char **argv, char **envp) {
       exit(-1);
    }
 
+   if (*filter) {
+      evaluate();
+      exit(0);
+   }
+
    // have a prompt only if the input is coming from a tty
    if (!isatty(fileno(stdin)) || !isatty(fileno(stdout)))
       opt_prompt = (char *) NULL;
