@@ -1,4 +1,4 @@
-//  $Id: route-list.cc 1.5.1.10.1.16 Wed, 19 May 1999 17:03:35 -0700 wlee $
+//  $Id: route-list.cc 1.5.1.10.1.21 Thu, 19 Apr 2001 23:16:41 -0700 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -90,7 +90,7 @@ char *RouteList::format(RouteList::Route *vr) {
 
    RouteList::Route::db_as* p;
    for (p = vr->db_as_l.head(); p; p = vr->db_as_l.next(p))
-      sprintf(line + strlen(line), "  %s:%-4d", p->db, p->as);
+      sprintf(line + strlen(line), "  %s:AS%d", p->db, p->as);
 
    return line;
 }
@@ -331,8 +331,9 @@ RouteList::Route *RouteList::get_registration_response() {
       routelist.vr_response_next = p;
       
       //      if (! RLWhois->Response(response))
-      if (!RLWhois->getSourceOrigin(response))
+      if (!RLWhois->getSourceOrigin(response)) {
 	 return vr;
+      }
 
       tcl_Eval(".statusbar.right configure -text {Pending Replies: %d}", 
 	       --irr_pending_replies);
@@ -377,6 +378,7 @@ RouteList::Route *RouteList::ask_registration() {
 void RouteList::get_registrations(ClientData clientdata) {
    char buffer[1024];
    RouteList::Route *vr;
+
    if (! routelist.vr_response_next && ! routelist.vr_query_next)
       return;
 
@@ -412,7 +414,7 @@ void RouteList::get_registrations(ClientData clientdata) {
    Tk_DoWhenIdle(get_registrations, (ClientData) NULL);
 }
 
-#define PENDING_REQ_WINDOW 45
+#define PENDING_REQ_WINDOW 1
 
 void RouteList::get_registrations_init(RouteList::Route *first) {
    RouteList::Route *vr1 = (RouteList::Route *) 1;
@@ -438,9 +440,9 @@ void RouteList::get_registrations_init(RouteList::Route *first) {
       if (IRR::GetProtocol() == IRR::ripe || IRR::GetProtocol() == IRR::bird)
          vr1 = ask_registration();
       else
-// Quick hack end here
-      for (int i = 0; i < PENDING_REQ_WINDOW && vr1; ++i)
-	 vr1 = ask_registration();
+	 // Quick hack end here
+	 for (int i = 0; i < PENDING_REQ_WINDOW && vr1; ++i)
+	    vr1 = ask_registration();
 
       // Changed by wlee@isi.edu
       //      get_registrations((ClientData) NULL);
@@ -487,15 +489,6 @@ void RouteList::load(ASt _as) {
 	i++)
      radix.insert(new RouteList::Route((*registered_routes)[i]));
 
-   /*
-   for (PrefixRange *prefixRange = registered_routes.first(); 
-	prefixRange; 
-	prefixRange = registered_routes.next()) {
-      vr = new RouteList::Route(*prefixRange);
-      radix.insert(vr);
-   }
-   */
-
    // below code is less eficient than above commented code
    // however user interface looks better
    // undo this once expand returns a RadixSet
@@ -540,12 +533,15 @@ void RouteList::read_bgp_dump(char *fname) {
       if (! radix.find(&pr)){
 	 vr = new RouteList::Route(pr);
 	 vr->rtd_status = VR_ROUTED_YES;
-
-	 if (! irr_radix->find(&vr->route)) {
-	    vr->reg_status = VR_REGISTERED_NO;
-	    vr->status = VR_IRR_DONT_QUERY;
+	 
+	 if (irr_radix->left || irr_radix->rght) { // has a node other than 0/0
+	    if (! irr_radix->find(&vr->route)) {
+	       vr->reg_status = VR_REGISTERED_NO;
+	       vr->status = VR_IRR_DONT_QUERY;
+	    }
+	 } else {
+	    vr->reg_status = VR_REGISTERED_UNDTRMND;
 	 }
-
 	 rlist.append(vr);
 	 radix.insert(vr);
       }
