@@ -1,4 +1,4 @@
-//  $Id: rptype.cc 1.2 Tue, 23 Nov 1999 11:26:03 -0800 cengiz $
+//  $Id: rptype.cc 1.4 Thu, 09 Nov 2000 15:57:20 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -44,7 +44,9 @@ class Attribute;
 #include "schema.hh"
 
 extern "C" {
+#ifndef HAVE_STRCASECMP_DEFINITION
 int strcasecmp(...);
+#endif // HAVE_STRCASECMP_DEFINITION
 }
 
 ////////////////////////////// RPType ////////////////////
@@ -52,6 +54,8 @@ int strcasecmp(...);
 RPType *RPType::newRPType(char *name) {
    if (!strcasecmp(name, "integer"))
       return new RPTypeInt;
+   if (!strcasecmp(name, "date"))
+      return new RPTypeDate;
    if (!strcasecmp(name, "real"))
       return new RPTypeReal;
    if (!strcasecmp(name, "rpsl_word"))
@@ -157,6 +161,22 @@ bool RPTypeInt::validate(const Item *item) const {
 	   && max >= ((ItemINT *) item)->i);
 }
 
+////////////////////////////// RPTypeDate ////////////////////
+
+bool RPTypeDate::validate(const Item *item) const {
+   static int days[] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+
+   if (typeid(*item) != typeid(ItemINT))
+      return false;
+
+   int year = ((ItemINT *) item)->i / 10000;
+   int month = (((ItemINT *) item)->i % 10000) / 100;
+   int day = ((ItemINT *) item)->i % 100;
+   return (year >= 1990 
+	   && month >= 01 && month <= 12
+	   && day >= 01 && day <= days[month - 1]);
+}
+
 ////////////////////////////// RPTypeReal ////////////////////
 
 bool RPTypeReal::validate(const Item *item) const {
