@@ -1,4 +1,4 @@
-//  $Id: schema_rpsl.cc 1.10 Tue, 30 May 2000 17:13:48 -0700 cengiz $
+//  $Id: schema_rpsl.cc 1.15 Mon, 12 Mar 2001 16:38:04 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -350,8 +350,6 @@ attr:  admin-c              mandatory,  multiple, lookup
 attr:  tech-c               mandatory,  multiple, lookup
 attr:  mnt-by               mandatory,  multiple, lookup " CMN_ATTRS "
 
-class: test
-attr: test syntax(special, peer)
 ";
 
 void Schema::addClass(AttrClass *clss) {
@@ -474,7 +472,7 @@ Object *Schema::initializeFromString(char *class_text, char *lookFor) {
       class_text = strstr(class_text, "\n\n");
       if (class_text)
 	 class_text += 2;
-   } while (class_text);
+   } while (class_text && *class_text);
 
    return result;
 }
@@ -522,6 +520,7 @@ static RPSLKeyword rpsl_keywords[] = {
 
    RPSLKeyword("syntax",       KEYW_SYNTAX,       1),
    RPSLKeyword("special",      KEYW_SPECIAL,      1),
+   RPSLKeyword("regexp",       KEYW_REGEXP,       1),
    RPSLKeyword("optional",     KEYW_OPTIONAL,     1),
    RPSLKeyword("mandatory",    KEYW_MANDATORY,    1),
    RPSLKeyword("internal",     KEYW_INTERNAL,     1),
